@@ -115,6 +115,9 @@ sub init {
         return if $_on_hold{$wid};
         $_on_hold{$wid} = AE::timer 0, 0.09, sub { exists $_on_hold{$wid} and delete $_on_hold{$wid} };
 
+        my $new_screen = screen_by_xy($evt->{root_x}, $evt->{root_y});
+        $new_screen->focus() if $focus->{screen} != $new_screen;
+
         $win->focus() if ($focus->{window} // 0) != $win;
     });
 
